#!/bin/bash
# Copyright 1999-2003 Gentoo Technologies, Inc.
# Distributed under the terms of the GNU General Public License v2
# $Header: /home/cvsroot/gentoo-projects/hardened-gcc/gentoo/distrib/2.4.5/noarch/hardened-gcc-common.sh,v 1.3 2003/11/09 16:28:30 pappy Exp $

HGCC_SCRIPT_EBUILD_REPLACEMENT_VERSION="hcc-2.4.5-x86"
HGCC_SCRIPT_EBUILD_REPLACEMENT_ARCHITECTURE="x86"

# the stack protection automatic setting, can also be -fstack-protector-all in newer releases
HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER="-q|-Q|--enable-propolice"
HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_LC="-q"
HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_UC="-Q"
HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_LONG="--enable-propolice"
HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_DESCRIPTION="enable automatic -fstack-protector in the transparent specs file"
HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE="-p|-P|--no-propolice"
HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE_LC="-p"
HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE_UC="-P"
HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE_LONG="--no-propolice"

SPECS_STD_EXCL_FRONT="%{!D__KERNEL__: %{!DIN_GCC: %{!static: %{!nostartfiles: %{!nodefaultlibs: %{!nostdlib: %{!nostdinc:"
SPECS_STD_EXCL_BACK="} } } } } } }"
SPECS_LINK_EXC_FRONT="%{!shared: %{!Bshareable: %{!i: %{!r:"
SPECS_LINK_EXC_BACK="} } } }"

SPECS_GCC_DIRECTORY="$(gcc --print-libgcc-file-name)"
SPECS_GCC_CONFIG_L="$(dirname ${SPECS_GCC_DIRECTORY})/specs"

SED_HELPER_FUNCTION="sed_specs_common"

sed_specs_common() {
	# ewarn "running sed_specs_common: [${REPLACESTRING}]"
	SPECS_LINK_SED_SOURCE=$(grep -A1 "^${SPECS_SECTION}$" ${SPECS_GCC_CONFIG_L} | tail -n 1)
	SPECS_LINK_SED_TARGET=$(echo ${SPECS_LINK_SED_SOURCE} | sed "${REPLACESTRING}")
	sed -i "s;${SPECS_LINK_SED_SOURCE};${SPECS_LINK_SED_TARGET};" ${SPECS_GCC_CONFIG_L}

	return 0
}

show_version() {
	echo -e "config: ${HGCC_SCRIPT_EBUILD_REPLACEMENT_VERSION}"
	echo -e "sys-devel/hardened-gcc is a compiler specs modification for gentoo-hardened"
	echo -e "made by Alexander Gabert <pappy@gentoo.org> and Ned Ludd <solar@gentoo.org>"
	echo -e "Distributed by Gentoo Linux - Copyright 1999-2003 Gentoo Technologies, Inc."
	echo -e "Distributed under the terms of the GNU General Public License, v2 or later!"
	return 0
}

show_current() {
	if [ "$TERM" == "xterm" ]
	then
		einfo " glibc: $(/lib/libc.so.6 | head -n 1)"
		einfo " glibc: $(/lib/libc.so.6 | grep '^Compiled by')"
		einfo "   gcc: $(gcc -v 2>&1| tail -n 1)"
		einfo "    ld: $(ld -v 2>&1| tail -n 1)\n"
		
		einfo "config: ${HGCC_SCRIPT_EBUILD_REPLACEMENT_VERSION}  -  ${HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE}"
		einfo "${HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_DESCRIPTION}: ${HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT}"
	else
		echo -e  " glibc: $(/lib/libc.so.6 | head -n 1)"
		echo -e  " glibc: $(/lib/libc.so.6 | grep '^Compiled by')"
		echo -e  "   gcc: $(gcc -v 2>&1| tail -n 1)"
		echo -e  "    ld: $(ld -v 2>&1| tail -n 1)\n"
		
		echo -e  "config: ${HGCC_SCRIPT_EBUILD_REPLACEMENT_VERSION}  -  ${HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE}"
		echo -e  "${HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_DESCRIPTION}: ${HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT}"
	fi
	
	# check for propolice on any arch other than hppa
	grep -B1 -i "yno_propolice" ${SPECS_GCC_CONFIG_L}
	grep -B1 -i "yet_exec" ${SPECS_GCC_CONFIG_L}
	
	return 0
}

restore_original_specs() {
	einfo "recovering original gcc specs (dumpspecs)"
	gcc -dumpspecs > ${SPECS_GCC_CONFIG_L}
	return 0
}

buzzer_warning() {
	echo
	echo
	ewarn "hcc-2.4.5-x86: if you notice problems with emerging applications"
	ewarn "hcc-2.4.5-x86: please make sure you disabled hcc before submitting"
	ewarn "hcc-2.4.5-x86: bugs to the gcc porting or gcc team, thank you!"
	ewarn "hcc-2.4.5-x86: when the problem remains only with hardened-gcc,"
	ewarn "hcc-2.4.5-x86: you have to contact gentoo-hardened directly at this URL:"
	einfo "hcc-2.4.5-x86: http://www.gentoo.org/proj/en/hardened/etdyn-ssp.xml"
	echo
	echo
	echo -ne "\a" ; sleep 0.1 &>/dev/null ; sleep 0,1 &>/dev/null
    echo -ne "\a" ; sleep 1
    echo -ne "\a" ; sleep 0.1 &>/dev/null ; sleep 0,1 &>/dev/null
	echo -ne "\a" ; sleep 1
	echo -ne "\a" ; sleep 0.1 &>/dev/null ; sleep 0,1 &>/dev/null
	echo -ne "\a" ; sleep 1
	echo -ne "\a" ; sleep 0.1 &>/dev/null ; sleep 0,1 &>/dev/null
	echo -ne "\a" ; sleep 1
	echo -ne "\a" ; sleep 0.1 &>/dev/null ; sleep 0,1 &>/dev/null
	echo -ne "\a" ; sleep 1
	sleep 8
	return 0
}

usage() {
	echo -e "Usage: hcc-2.4.5-x86 [single option]"
	echo -e "Change the current cc/gcc specs file for etdyn and propolice\n"
	echo -e "\t\t-l, -L, --list"
	echo -e "\t\t\tlist currently active settings\n"
	echo -e "\t\t-a, -A, --activate"
	echo -e "\t\t\tactivate default transparent settings\n"
	echo -e "\t\t-r, -R, --restore"
	echo -e "\t\t\trecover original settings\n"
	echo -e "\t\t-v, -V, --version"
	echo -e "\t\t\tdisplay program version\n"

	echo -e "\t\t${HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER//|/ }"
	echo -e "\t\t\t${HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_DESCRIPTION}\n"

	echo -e "\t\t${HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE//|/ }"
	echo -e "\t\t\t${HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE_DESCRIPTION}\n"

	return 0
}

NEED_ACTION="yes"
DOIT="usage"

for x in $*
	do
		case "${x}" in
			"${HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_LC}"| \
			"${HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_UC}"| \
			"${HGCC_ENABLE_AUTO_PROPO_PREPROC_ENABLER_LONG}")
				if [ "${NEED_ACTION}" = "yes" ]
				then
					NEED_ACTION="no"
					einfo "activating HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT"
					sed -i 's:HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT="no":HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT="yes":g' \
						${HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE}
					DOIT="activate_transparent"
					
				fi
				;;
			"${HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE_LC}"| \
			"${HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE_UC}"| \
			"${HGCC_ENABLE_AUTO_PROPO_PREPROC_DISABLE_LONG}")
				if [ "${NEED_ACTION}" = "yes" ]
				then
					NEED_ACTION="no"
					ewarn "disabling HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT"
					sed -i 's:HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT="yes":HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT="no":g' \
						${HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE}
					DOIT="activate_transparent"
				fi
				;;
			-l|-L|--list)
				if [ "${NEED_ACTION}" = "yes" ]
				then
					NEED_ACTION="no"
					DOIT="show_current"
				fi
				;;
			-v|-V|--version)
				if [ "${NEED_ACTION}" = "yes" ]
				then
					NEED_ACTION="no"
					DOIT="show_version"
				fi
				;;
			-a|-A|--activate)
				if [ "${NEED_ACTION}" = "yes" ]
				then
					NEED_ACTION="no"
					eval "buzzer_warning"
					DOIT="activate_transparent"
				fi
				;;
			-r|-R|--restore)
				if [ "${NEED_ACTION}" = "yes" ]
				then
					NEED_ACTION="no"
					DOIT="restore_original_specs"
				fi
				;;
			-*)
				NEED_ACTION="no"
				DOIT="usage"
				;;
		esac
	done

#EOF#
